/*
	Program that computes the homography 
	given a set of points and their relation

	Sergio Herrera July/2018
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "pnm.h"
#include "fill.h"
#include <float.h>
#include <math.h>

#include "morphology.h"
#include "follow.h"
#include "chamfer.h"
#include "triangles.h"

//#include "matrices.h"
//#include "eigen.h"

#include "singular_value_decomposition.h"              
//#include <clapack.h>


#define CINC 5

int do_work( char *namein );
void negate( MATRIX* );
void umbral( MATRIX* , int );
void findOBJ( const MATRIX *IMG , int* X, int* Y , int COLOR);

void cleanBorder( MATRIX *IN, int R);

void descomposicionQR( double **A, int m, int n, double **R, double *vec );
void resuelve( double **Rp, int n, double *x );
void matrix_mul(double ** A,double ** B,double ** C);
void matrix_MUL(double ** A,int m, int n, double ** B, int p, int q, double ** C);
int svd(double** A, double** U, double** D, double** V );
void CalculaR( double **A, double **R  );


extern MATRIX * convolution ( MATRIX *IN, MATRIX *FIL, int ka );


int main( int argc, char *argv[] )
{
	char namein[64];

	if ( argc < 2 ) { 
		fprintf( stderr, "  [!]Use: %s input_image.pbm \n", argv[0]);
		return -1;
	}

	strncpy( namein, argv[1], 64 );
	
	if ( do_work( namein ) >= 0 ) {
		printf("\n\n# Program Done! \n");
		//fprintf( stderr, "# Done!!! \n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}

	return 0;
}

int do_work( char *namein ) 
{
	MATRIX *IMG, *IN, *FIL;

	IN = read_pnm( namein ); 

	if ( IN ==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in b/w\n");
		return -1;
	}
	
	//2 - Smoothing (2 Pass)
		int **fil;
		FIL = matrix_allocate( 3, 3, sizeof( int ) );
		if ( FIL==NULL ) {
			fprintf( stderr, "ERROR allocating memory\n" );
			return -1;
		}
		//Smooth Window
		fil = (int **)FIL->ptr;
		fil[0][0] = 1; fil[0][1] = 1; fil[0][2] = 1;
		fil[1][0] = 1; fil[1][1] = 1; fil[1][2] = 1;
		fil[2][0] = 1; fil[2][1] = 1; fil[2][2] = 1;

		IMG = convolution( IN, FIL, 9 );	//1st Pass
				write_pnm( IMG, "1_convol1.pgm" );

		//memcpy( IN, IMG, sizeof(MATRIX) );
		//IMG = convolution( IN, FIL, 9 );	//2nd Pass
				//write_pnm( IMG, "convol2.pgm" );

	
	//3 - Binarize IMG global threshold 
		umbral( IMG, 40 );
			write_pnm( IMG, "2_threshold.pgm" );

	//4 - Opening / Erotion->Dilatation
		int Umbral = 13;
		//Opening
			//Erosion
			negate( IMG );
			chamfer34( IMG ); 
			umbral( IMG , Umbral );
			negate( IMG );
					write_pnm( IMG, "3_EROTION.pgm" );
			//Dilatation
			chamfer34( IMG );
			umbral( IMG , Umbral );
					write_pnm( IMG, "4_DILATATION.pgm" );
			cleanBorder( IMG , Umbral );
					write_pnm( IMG, "5_OPENING.pgm" );

	//5 - Extract triangles (Fill)
		printf("#Looking for objects... \n");
		int X,Y;
		int OBJcount = 0;
		int OBJ[5];
		int COLOR;
		
		//Look for objects to fill them
		while(1)
		{
			findOBJ(IMG, &X, &Y, 0);
			if(X > -1){ //Valid Point
				//printf("#Found object at: %d , %d \n", X,Y);
				COLOR = FILLTONE + OBJcount * CINC;
				
				push(X,Y);
				do_fill(IMG, OBJ , 0 , COLOR );	//Fill 0's with COLOR

				OBJcount+=1;
			}
			else break;
		}
		printf("# \t [%d] Objects found! \n", OBJcount);
		if(!OBJcount) return( -1 );

				write_pnm( IMG, "6_Segmentation.pgm" );		


	//Save each Scene vertex in a list
	DPOINT* PointS = malloc( sizeof(DPOINT) * 3 * OBJcount );
 
	//6 - Compute vertices of each triangle
		printf("# \n" );

		//Allocate memory enough for ANY perimeter segment
		POINT *perimeter = malloc( sizeof(POINT) * 2000 );
		//Allocate memory for each triangle vertices
		POINT* vertices = malloc( sizeof(POINT) * 3 );
		DPOINT* dv = malloc( sizeof(DPOINT) * 3 );

		for (int i = 0; i < OBJcount; ++i)
		{
			COLOR = FILLTONE + i * CINC;

			//a) Find first pixel of OBJColor
			findOBJ(IMG, &X, &Y, COLOR); //Find first vertex
			if(X > -1){ //Valid Point
				printf("\n\t ### Object [%d] ###\n\n",i+1);

				//Get Perimeter
				push(X,Y);
				do_fill(IMG, OBJ , COLOR , 1 );	//Fill COLOR's with 1's
						//write_pnm( IMG, "Fill.pgm" );		
				dilation_4neig_inplace_obj( IMG , OBJ, 1 );
						//write_pnm( IMG, "perimetroD.pgm" );		

				//Convert perimeter
				byte *pimg;
				for(int k=OBJ[INICIOY]; k<=OBJ[FINY]; k++ ) {
					pimg = (byte *)IMG->ptr[k] + OBJ[INICIOX];
					for(int p=OBJ[INICIOX]; p<=OBJ[FINX]; p++ ) {
						if ( *pimg == 254 ) {
							*pimg = 200;
						}
						pimg++;
					}
				}
				//write_pnm( IMG, "perimetroE.pgm" );		

		
				//printf("\n Size of POINT:: %d \n", sizeof(POINT) );
				
				int Counter = 0;

				follow_perimeter( IMG , Y, X , 200,    &Counter,   perimeter);
				//printf("#Counter: %d \n",Counter);
						write_pnm( IMG, "8_Perimeter.pgm" );		
				/*
				printf("\t OUTSIDE: Pixels of segment\n");
				for (int k = 0; k <= Counter ; k++)
				{
					printf(" %d,%d \t", perimeter[k].x, perimeter[k].y );
				}
				//*/

				//Extract triangles
				
				extrae_triangulo( perimeter , Counter , vertices, dv );
				
				//printf("#Triangle extracted...\n");

				printf("#\t     %d,%d \t %d,%d \t %d,%d \t#NEW\n", vertices[0].x , vertices[0].y,
																vertices[1].x , vertices[1].y,
																vertices[2].x , vertices[2].y);

				printf("#\t %.02f,%.02f \t %.02f,%.02f \t %.02f,%.02f \t#FLOAT\n", dv[0].x , dv[0].y,
																dv[1].x , dv[1].y,
																dv[2].x , dv[2].y);
										write_pnm( IMG, "10_FINAL.pgm" );		

				//Save vertices in list
				PointS[ 3*i ].x = dv[0].x;		PointS[ 3*i ].y = dv[0].y;  
				PointS[3*i+1].x = dv[1].x;		PointS[3*i+1].y = dv[1].y;
				PointS[3*i+2].x = dv[2].x;		PointS[3*i+2].y = dv[2].y;

			}
		}

		printf("\n\n#\t Scene [P] Vertex List:\n");
		for (int i = 0; i < OBJcount; ++i)
		{
			printf("#\t %.02f,%.02f \t %.02f,%.02f \t %.02f,%.02f \t\t #TRI:%d \n", PointS[ 3*i ].x , PointS[ 3*i ].y,
																				 PointS[3*i+1].x , PointS[3*i+1].y,
																				 PointS[3*i+2].x , PointS[3*i+2].y, i+1);
				
		}

		//Inverting y's of points 
		for (int i = 0; i < 6; ++i)
		{
			PointS[i].y = 480 - PointS[i].y;
		}

		printf("\n#\t INVERTED Scene [P] Vertex List:\n");
		for (int i = 0; i < OBJcount; ++i)
		{
			printf("#\t %.02f,%.02f \t %.02f,%.02f \t %.02f,%.02f \t\t #TRI:%d \n", PointS[ 3*i ].x , PointS[ 3*i ].y,
																				 PointS[3*i+1].x , PointS[3*i+1].y,
																				 PointS[3*i+2].x , PointS[3*i+2].y, i+1);
				
		}

	//Compute Homography
		
		//Img Points
		DPOINT p[6], pORG[6];
		//MARKER
		p[0].x = 0;		p[0].y = 0;
		p[1].x = 255;	p[1].y = 0;
		p[2].x = 255;	p[2].y = 255;
		p[3].x = 0;		p[3].y = 255;
		p[4].x = 30;	p[4].y = 100;
		p[5].x = 130;	p[5].y = 90;

		//MARKER ORIGINAL
		pORG[0].x = 0;		pORG[0].y = 0;
		pORG[1].x = 255;	pORG[1].y = 0;
		pORG[2].x = 255;	pORG[2].y = 255;
		pORG[3].x = 0;		pORG[3].y = 255;
		pORG[4].x = 30;		pORG[4].y = 100;
		pORG[5].x = 130;	pORG[5].y = 90;	

		//ORDER OF POINTS
		short order[6] = {1,0,5,4,3,2}; //IMG1
		//short order[6] = {0,2,5,4,3,1};	//IMG2
		//short order[6] = {0,2,5,4,3,1};	//IMG3
		//short order[6] = {1,0,5,4,3,2};	//IMG4
		//short order[6] = {0,2,5,4,3,1};	//IMG5

		//for (int i = 0; i < 6; ++i)
		//	printf("Order[%d]= %d \n",i,order[i]);

		//Print relation of points
		printf("\n");
		for (int i = 0; i < 6; ++i)
			printf(" \t %d - [%.02f,%.02f] -> [%.04f,%.04f] \n", i ,p[i].x,p[i].y,   PointS[order[i]].x, PointS[order[i]].y  );

		//Matrix T1 and T2
		MATRIX* MT1 = matrix_allocate( 3, 3, sizeof(double) );
		MATRIX* MT2 = matrix_allocate( 3, 3, sizeof(double) );
		if( MT1==NULL || MT2==NULL ) {
			fprintf( stderr, "ERROR no hay memoria para la matrices T1 y T1\n" );
			return( -3 );
		}
		double ** T1 = (double **)MT1->ptr;
		double ** T2 = (double **)MT2->ptr;

		//Compute median and variance of P (scene points)
			double medianX = 0;
			double medianY = 0;
			for (int i = 0; i < 6; ++i)
			{
				medianX += PointS[ i ].x;
				medianY += PointS[ i ].y;
			}
			medianX /= 6;
			medianY /= 6;
			printf("\n\n# Median of [PointS]: %f , %f \n", medianX, medianY);

			double varX, varY;
			varX = varY = 0;
			for (int i = 0; i < 6; ++i)
			{
				varX += (PointS[i].x - medianX) * (PointS[i].x - medianX);
				varY += (PointS[i].y - medianY) * (PointS[i].y - medianY);
			}
			varX /= 6;
			varY /= 6;
			printf("#variance of [PointS]: %f , %f \n", varX, varY);

			//Normalize vertices in image
			for (int i = 0; i < 6; ++i)
			{
				PointS[i].x = (PointS[i].x - medianX ) /varX;
				PointS[i].y = (PointS[i].y - medianY ) /varY;
			}
			//Build T1^-1 (inverse)
			T1[0][0] = varX;	T1[0][1] = 0; 		T1[0][2] = medianX;
			T1[1][0] = 0;		T1[1][1] = varY; 	T1[1][2] = medianY;
			T1[2][0] = 0;		T1[2][1] = 0; 		T1[2][2] = 1;
			
			/*
			//Build T2
			T2[0][0] = 1/varX;	T2[0][1] = 0; 		T2[0][2] = -medianX/varX;
			T2[1][0] = 0;		T2[1][1] = 1/varY; 	T2[1][2] = -medianY/varY;
			T2[2][0] = 0;		T2[2][1] = 0; 		T2[2][2] = 1;
			*/
			printf("\n# T1 \n");
			for (int i = 0; i < 3; ++i)
			{
				for (int j = 0; j < 3; ++j)
				{
					printf(" %f ", T1[i][j] );
				}
				printf("\n");
			}

			printf("\n\t#Normalized [PointS] Vertex List:\n");
			for (int i = 0; i < OBJcount; ++i)
			{
				printf("#\t %.04f,%.04f \t %.04f,%.04f \t %.04f,%.04f \t #TRI:%d \n", PointS[ 3*i ].x , PointS[ 3*i ].y,
																					 PointS[3*i+1].x , PointS[3*i+1].y,
																					 PointS[3*i+2].x , PointS[3*i+2].y, i+1);
			}

		//Compute median and variance of p (marker/img points)
			medianX = 0;
			medianY = 0;
			for (int i = 0; i < 6; ++i)
			{
				medianX += p[ i ].x;
				medianY += p[ i ].y;
			}
			medianX /= 6;
			medianY /= 6;
			printf("\n\n#  Median of [p]: %f , %f \n", medianX, medianY);

			varX = varY = 0;
			for (int i = 0; i < 6; ++i)
			{
				varX += (p[i].x - medianX) * (p[i].x - medianX);
				varY += (p[i].y - medianY) * (p[i].y - medianY);
			}
			varX /= 6;
			varY /= 6;
			printf(" Variance of [p]: %f , %f \n", varX, varY);

			//Normalize vertices in image
			for (int i = 0; i < 6; ++i)
			{
				p[i].x = (p[i].x - medianX)/varX;
				p[i].y = (p[i].y - medianY)/varY;
			}
			
			//Build T2
			T2[0][0] = 1/varX;	T2[0][1] = 0; 		T2[0][2] = -medianX/varX;
			T2[1][0] = 0;		T2[1][1] = 1/varY; 	T2[1][2] = -medianY/varY;
			T2[2][0] = 0;		T2[2][1] = 0; 		T2[2][2] = 1;
			/*
			//Build T1^-1 (inverse)
			T1[0][0] = varX;	T1[0][1] = 0; 		T1[0][2] = medianX;
			T1[1][0] = 0;		T1[1][1] = varY; 	T1[1][2] = medianY;
			T1[2][0] = 0;		T1[2][1] = 0; 		T1[2][2] = 1;
			*/

			printf("\n# T2 \n");
			for (int i = 0; i < 3; ++i)
			{
				for (int j = 0; j < 3; ++j)
				{
					printf(" %f ", T2[i][j] );
				}
				printf("\n");
			}

			printf("\n\n#\tNormalized [p] Vertex List:\n");
			for (int i = 0; i < OBJcount; ++i)
			{
				printf("#\t %.04f,%.04f \t %.04f,%.04f \t %.04f,%.04f \t #TRI:%d \n", p[ 3*i ].x , p[ 3*i ].y,
																					 p[3*i+1].x , p[3*i+1].y,
																					 p[3*i+2].x , p[3*i+2].y, i+1);
			}

		
		//Build Matrix B = [A|b]
		MATRIX *MB,*MR;
		double **R, **B;

		MB = matrix_allocate( 12, 9, sizeof(double) );
		MR = matrix_allocate( 9, 9, sizeof(double) );
		if( MB==NULL || MR==NULL ) {
			fprintf( stderr, "ERROR no hay memoria para la matrices MB y MR\n" );
			return( -3 );
		}
		B = (double **)MB->ptr;
		R = (double **)MR->ptr;



		for (int i = 0; i < 12; ++i)
		{
			for (int j = 0; j < 9; ++j)
			{
				B[i][j] = -1; 
			}
		}

		/*
		//Print B
		printf("B=\n");
		for (int i = 0; i < 12; ++i)
		{
			for (int j = 0; j < 9; ++j)
			{
				printf("  [%d,%d]= %.04f",i,j, B[i][j] ); 
			}
			printf("\n");
		}
		//*/

		printf("\n# >>Building matrix B = [A|b] \n");
		for (int k = 0; k < 6; ++k)
		{	
			//Build A
			short i;
			//(2*k)-th row
			i = 2*k;
			B[i][0] = p[k].x;	B[i][1] = p[k].y;	B[i][2] = 1;  
			B[i][3] = 0;	B[i][4] = 0;	B[i][5] = 0;  
			B[i][6] = -p[k].x * PointS[order[k]].x;	B[i][7] = -p[k].y * PointS[order[k]].x; /**/
				//build b
				B[i][8] = PointS[order[k]].x;
			
			//(2*k+1)-th row
			i = 2*k+1;
			B[i][0] = 0;	B[i][1] = 0;	B[i][2] = 0;  
			B[i][3] = p[k].x;	B[i][4] = p[k].y;	B[i][5] = 1;  
			B[i][6] = -p[k].x * PointS[order[k]].y;	B[i][7] = -p[k].y * PointS[order[k]].y; /**/
				//build b
				B[i][8] = PointS[order[k]].y;
		}


			printf("#B=\n");
			for (int i = 0; i < 12; ++i)
			{
				for (int j = 0; j < 9; ++j)
				{
					//printf("  [%d,%d]= %.04f",i,j, B[i][j] ); 
					printf(" %.04f \t", B[i][j] ); 
				}
				printf("\n");
			}


		//QR Factorization
			printf("\n# >>QR Factorization\n");

			double * H;
			H = (double *)malloc( 12*sizeof(double) );
			//for (int i = 0; i < 12; ++i)
			//	printf("[%d]=%.4f \t",i,H[i] );

			descomposicionQR( B, 12 , 9 , R, H );

			printf("\t# R[9x9]=\n");
			for (int i = 0; i < 9; ++i)
			{
				for (int j = 0; j < 9; ++j)
				{
					//printf("  [%d,%d]= %.04f",i,j, B[i][j] ); 
					printf(" %.04f \t", R[i][j] ); 
				}
				printf("\n");
			}

		//Compute H
		printf("\n# >>Solving by backward substitution\n");
		resuelve( R , 8 , H);

		printf("\t# H[]=\n");
		for (int i = 0; i < 9; ++i)
			(i==8)?printf("|| [%d]=%.4f ",i,H[i] ):printf(" [%d]=%.4f ",i,H[i] );

		//Normalize Homography
			MATRIX* Mres = matrix_allocate( 3, 3, sizeof(double) );
			MATRIX* MHn = matrix_allocate( 3, 3, sizeof(double) );
			if( Mres==NULL || MHn==NULL ) {
				fprintf( stderr, "ERROR no hay memoria para la matrices T1 y T1\n" );
				return( -3 );
			}
			double ** r = (double **)Mres->ptr;
			double ** Hn = (double **)MHn->ptr;
			
			/*//Print blank H
			printf("\n   [ H ] \n");
			for (int i = 0; i < 3; ++i)
			{
				for (int j = 0; j < 3; ++j)
				{
					printf(" %f ", Hn[i][j] );
				}
				printf("\n");
			}
			*/

			//Copy homography array to matrix
			Hn[0][0] = H[0];	Hn[0][1] = H[1];	Hn[0][2] = H[2];
			Hn[1][0] = H[3];	Hn[1][1] = H[4]; 	Hn[1][2] = H[5];
			Hn[2][0] = H[6];	Hn[2][1] = H[7]; 	Hn[2][2] = 1;

			printf("\n#   [ H ] \n");
			for (int i = 0; i < 3; ++i)
			{
				for (int j = 0; j < 3; ++j)
				{
					printf(" %f ", Hn[i][j] );
				}
				printf("\n");
			}

			matrix_mul( T1 , Hn , r);
			matrix_mul( r , T2 , Hn );

			printf("\n#   [ H_norm ] \n");
			for (int i = 0; i < 3; ++i)
			{
				for (int j = 0; j < 3; ++j)
				{
					printf(" %f ", Hn[i][j] );
				}
				printf("\n");
			}

			//Copy back homography array from matrix
			H[0] = Hn[0][0];	H[1] = Hn[0][1];	H[2] = Hn[0][2];
			H[3] = Hn[1][0];	H[4] = Hn[1][1]; 	H[5] = Hn[1][2];
			H[6] = Hn[2][0];	H[7] = Hn[2][1]; 	H[8] = Hn[2][2];

			//Project original points
				MATRIX *MPX = matrix_allocate( 4, 1, sizeof(double) );
				MATRIX *MPR = matrix_allocate( 3, 1, sizeof(double) );
				if( MPX == NULL || MPR == NULL ) {
					fprintf( stderr, "ERROR no hay memoria para la matrices [MPX/MPR] \n" );
					return( -15 );
				}
				double ** PX = (double **)MPX->ptr;
				double ** pRes = (double **)MPR->ptr;

				printf("\n# INVERTED HOMOGRAPHY rounded points\n ");
				for (int i = 0; i < 6; ++i)
				{
					//printf("Point = %f , %f \n", pORG[i].x, pORG[i].y);
					PX[0][0] = pORG[i].x; 
					PX[1][0] = pORG[i].y;  
					PX[2][0] = 1;
					//PX[3][0] = 1;

					matrix_MUL( Hn ,3,3, PX, 3,1, pRes );

					//Normalize
					for (int i = 0; i < 3; ++i)
						pRes[i][0] = pRes[i][0]/pRes[2][0];

					//printf("\t (%d) [ %.02f , %.02f , %.02f  ] \n", i , pRes[0][0], pRes[1][0], pRes[2][0] );
					printf("\t %d,%d \n", (int)floor(pRes[0][0]+0.5) , (int)floor((480-pRes[1][0])+0.5) );

				}













		//Compute f
			double u = 640 / 2.0; // 0 **
			double v = 480 / 2.0;
			double ff = ( u * ( H[1]*H[6] + H[7]*H[0] - u*H[7]*H[6] ) 
						+ v * ( H[4]*H[6] + H[7]*H[3] - v*H[7]*H[6] )
						- H[1]*H[0] - H[4]*H[3] ) 
						/ H[7]*H[6] ;
			double f = sqrt(ff);

			printf("\n\t f^2= %f \n", ff );
			printf("\n\t f= %f \n", f );

		//Compute Matrix K
			MATRIX* MK = matrix_allocate( 3, 3, sizeof(double) );
			MATRIX* MKi = matrix_allocate( 3, 3, sizeof(double) );
			if( MK == NULL || MKi == NULL ) {
				fprintf( stderr, "ERROR no hay memoria para la matrices MK y MKi\n" );
				return( -5 );
			}
			double ** K = (double **)MK->ptr;
			double ** Ki = (double **)MKi->ptr;
		
			//Build matrix K
				K[0][0] = f;	K[0][1] = 0;	K[0][2] = -u;
				K[1][0] = 0;	K[1][1] = f; 	K[1][2] = -v;
				K[2][0] = 0;	K[2][1] = 0; 	K[2][2] = -1;

			//Build matrix K^-1
				Ki[0][0] = 1/f;	Ki[0][1] = 0;	Ki[0][2] = -u/f;
				Ki[1][0] = 0;	Ki[1][1] = 1/f; Ki[1][2] = -v/f;
				Ki[2][0] = 0;	Ki[2][1] = 0; 	Ki[2][2] = -1;
		
			printf("\n K[]= \n");
			for (int i = 0; i < 3; ++i)
			{
				for (int j = 0; j < 3; ++j)
					printf(" %f ", K[i][j] );
				printf("\n");
			}	

			printf("\n K^-1[]= \n");
			for (int i = 0; i < 3; ++i)
			{
				for (int j = 0; j < 3; ++j)
					printf(" %f ", Ki[i][j] );
				printf("\n");
			}

		//Computing G = K^-1 * H	
			MATRIX* MG = matrix_allocate( 3, 3, sizeof(double) );
			if( MG == NULL ) {
				fprintf( stderr, "ERROR no hay memoria para la matrices MG \n" );
				return( -7 );
			}
			double ** G = (double **)MG->ptr;

			matrix_mul( Ki , Hn , G);


			printf("\n G = K^-1 * H = \n");
			for (int i = 0; i < 3; ++i)
			{
				for (int j = 0; j < 3; ++j)
				{
					printf(" %f ", G[i][j] );
				}
				printf("\n");
			}

	//Computing Lambda1 and Lamda2
		double Lambda1 = sqrt( G[0][0]*G[0][0] + G[1][0]*G[1][0] + G[2][0]*G[2][0] );
		double Lambda2 = sqrt( G[0][1]*G[0][1] + G[1][1]*G[1][1] + G[2][1]*G[2][1] );
		double Lambda  = (Lambda1 + Lambda2) / 2.0;
		printf("\n\t >>Lambda= %f \n", Lambda );

	//Saving t
			double t [3];
			t[0] = G[0][2]/Lambda;
			t[1] = G[1][2]/Lambda;
			t[2] = G[2][2]/Lambda;

			printf("\n\t t[]= \n");
			for (int i = 0; i < 3; ++i)
				printf("\t  %f \n", t[i] );

	//Computing r1 x r2
		MATRIX* MRd = matrix_allocate( 3, 3, sizeof(double) );
		MATRIX* MUU = matrix_allocate( 3, 3, sizeof(double) );
		MATRIX* MD = matrix_allocate( 3, 1, sizeof(double) );
		MATRIX* MVT = matrix_allocate( 3, 3, sizeof(double) );
		if( MRd == NULL || MUU == NULL || MD == NULL || MVT == NULL) {
			fprintf( stderr, "ERROR no hay memoria para la matrices UDV^T \n" );
			return( -11 );
		}
		double ** Rd = (double **)MRd->ptr;
		double ** UU = (double **)MUU->ptr;
		double ** D = (double **)MD->ptr;
		double ** VT = (double **)MVT->ptr;

		//R1							//R2							// R1 x R2
		Rd[0][0] = G[0][0]/Lambda;		Rd[0][1] = G[0][1]/Lambda;		Rd[0][2] = (G[1][0]*G[2][1] - G[2][0]*G[1][1])/Lambda;  
		Rd[1][0] = G[1][0]/Lambda;		Rd[1][1] = G[1][1]/Lambda;		Rd[1][2] = (G[2][0]*G[0][1] - G[0][0]*G[2][1])/Lambda;
		Rd[2][0] = G[2][0]/Lambda;		Rd[2][1] = G[2][1]/Lambda;		Rd[2][2] = (G[0][0]*G[1][1] - G[1][0]*G[0][1])/Lambda;


		printf("\n R' = \n");
		for (int i = 0; i < 3; ++i)
		{
			for (int j = 0; j < 3; ++j)
			{
				printf(" %f ", Rd[i][j] );
			}
			printf("\n");
		}


		//Compute R'= UDV^T 
			double* dummy = (double*) malloc(3 * sizeof(double));                  
			int err = Singular_Value_Decomposition( Rd , 3, 3, UU, *D, VT, dummy);		
			free(dummy);                                                     
	     	if (err < 0) printf(" Failed to converge\n"); 
	     	       printf("\n [D]= \n");
					for (int i= 0; i< 3; i++ )
						printf(" %lf ", D[ i ][0] );

					printf("\n\n [U]= \n");
					for (int i= 0; i< 3; i++ ) {
						for (int j= 0; j< 3; j++ )
							printf("%lf ", UU[i][j] );
						printf("\n");
					}

					printf("\n [V]= \n");
					for (int i= 0; i< 3; i++ ) {
						for (int j= 0; j< 3; j++ )
							printf("%lf ", VT[i][j] );
						printf("\n");
					}                


		//Transpose V
			double aux;
			aux = VT[0][1]; 	VT[0][1] = VT[1][0]; 	VT[1][0] = aux;
			aux = VT[0][2]; 	VT[0][2] = VT[2][0]; 	VT[2][0] = aux;
			aux = VT[1][2]; 	VT[1][2] = VT[2][1]; 	VT[2][1] = aux;

			printf("\n [V^T]= \n");
				for (int i= 0; i< 3; i++ ) {
					for (int j= 0; j< 3; j++ )
						printf("%lf ", VT[i][j] );
					printf("\n");
				}

		//Compute R = U*V^T
		matrix_mul( UU , VT , Rd );

		printf("\n\n ------------------- \n");
		printf("\n R = U * V^T \n");
		for (int i = 0; i < 3; ++i)
		{
			for (int j = 0; j < 3; ++j)
			{
				printf(" %f ", Rd[i][j] );
			}
			printf("\n");
		}

		printf("\n\t t[]= \n");
		for (int i = 0; i < 3; ++i)
				printf("\t  %f \n", t[i] );

		printf("************\n");
		
		printf("\n K[]= \n");
		for (int i = 0; i < 3; ++i)
		{
			for (int j = 0; j < 3; ++j)
			{
				printf(" %f ", K[i][j] );
			}
			printf("\n");
		}

		//*******
		//Building augmented matrix [R|t]
		MATRIX *MRT = matrix_allocate( 3, 4, sizeof(double) );
		MATRIX *MM = matrix_allocate( 3, 4, sizeof(double) );
		if( MRT == NULL || MM == NULL) {
			fprintf( stderr, "ERROR no hay memoria para la matrices [R|t] \n" );
			return( -14 );
		}
		double ** RT = (double **)MRT->ptr;
		double ** M = (double **)MM->ptr;

		//Build matrix [R|t]
		for (int i = 0; i < 3; ++i)
		{	
			int j;
			for (j = 0; j < 3; ++j)
				RT[i][j] = Rd[i][j];
			
			RT[i][j] = t[i];
		}


		printf("\n [R|t] \n");
		for (int i = 0; i < 3; ++i)
		{
			for (int j = 0; j < 4; ++j)
			{
				printf(" %f ", RT[i][j] );
			}
			printf("\n");
		}

		matrix_MUL( K ,3,3, RT, 3,4, M );


		printf("\n M = K[R|t] \n");
		for (int i = 0; i < 3; ++i)
		{
			for (int j = 0; j < 4; ++j)
			{
				printf(" %f ", M[i][j] );
			}
			printf("\n");
		}



		//Converting points
			


			printf("\n Projecting original points...\n ");
			for (int i = 0; i < 6; ++i)
			{
				//printf("Point = %f ,%f \n", pORG[i].x, pORG[i].y);
				PX[0][0] = pORG[i].x; 
				PX[1][0] = pORG[i].y;  
				PX[2][0] = 0;
				PX[3][0] = 1;

				matrix_MUL( M ,3,4, PX, 4,1, pRes );

				//Normalize
				for (int i = 0; i < 3; ++i)
					pRes[i][0] = pRes[i][0]/pRes[2][0];

				printf("\t (%d) [ %.02f , %.02f , %.02f ] \n", i , pRes[0][0], pRes[1][0], pRes[2][0] );
			}

			printf("\n INVERTED rounded points\n ");
			for (int i = 0; i < 6; ++i)
			{
				//printf("Point = %f ,%f \n", pORG[i].x, pORG[i].y);
				PX[0][0] = pORG[i].x; 
				PX[1][0] = pORG[i].y;  
				PX[2][0] = 0;
				PX[3][0] = 1;

				matrix_MUL( M ,3,4, PX, 4,1, pRes );

				//Normalize
				for (int i = 0; i < 3; ++i)
					pRes[i][0] = pRes[i][0]/pRes[2][0];

				printf("\t %d,%d \n", (int)floor(pRes[0][0]+0.5) , (int)floor((480-pRes[1][0])+0.5) );
			}

			/*
			printf("\n Projecting cube points...\n ");
			for (int i = 0; i < 6; ++i)
			{
				//printf("Point = %f ,%f \n", pORG[i].x, pORG[i].y);
				PX[0][0] = pORG[i].x; 
				PX[1][0] = pORG[i].y;  
				PX[2][0] = .3;
				PX[3][0] = 1;

				matrix_MUL( M ,3,4, PX, 4,1, pRes );

				//Normalize
				for (int i = 0; i < 3; ++i)
					pRes[i][0] = pRes[i][0]/pRes[2][0];

				//printf("\t (%d) [ %.02f , %.02f , %.02f ] \n", i , pRes[0][0], pRes[1][0], pRes[2][0] );
				printf("\t %d,%d \n", (int)floor(pRes[0][0]+0.5) , (int)floor((480-pRes[1][0])+0.5) );

			}
			*/





		matrix_free( MPX ); matrix_free( MPR );

		matrix_free( MRT );	matrix_free( MM );

		matrix_free( MUU );matrix_free( MD );matrix_free( MVT );
		matrix_free( MG );  matrix_free( MRd );
		matrix_free( MK );  matrix_free( MKi );
		matrix_free( MHn );	matrix_free( Mres );

		matrix_free( MT1 );
		matrix_free( MT2 );
		matrix_free( MB );
		matrix_free( MR );

		free(H);
		free(vertices); free(dv);

		free(perimeter);
		free(PointS);


	//7 - Output points

	//if ( write_pnm( IMG, "fillTest1.pgm" ) != 0 )  
    //    return -1;

	matrix_free( IMG );
	matrix_free( IN );
	matrix_free( FIL );

	return 0;
}

// Matrix Multiplication
//
void matrix_MUL(double ** A,int m, int n, double ** B, int p, int q, double ** C){
	if (n != p)
    printf("The matrices can't be multiplied with each other.\n");	

	int i, j, k;
	double sum = 0;

	for (i = 0; i < m; i++) {
      for (j = 0; j < q; j++) {
        for (k = 0; k < p; k++) {
          sum = sum + A[i][k] * B[k][j];
        }
 
        C[i][j] = sum;
        sum = 0;
      }
    }

}

// 3x3 Matrix Multiplication
//
void matrix_mul(double ** A,double ** B,double ** C){

   for (int i = 0; i < 3; i++) {
      for (int j = 0; j < 3; j++) {
         C[i][j] = 0;
         for (int k = 0; k < 3; k++)
            C[i][j] = C[i][j] + A[i][k] * B[k][j];
      }
   }
}


/**
  Subrutina que calcula descomposición QR
  A = QR, A es de tamaño m x n
  No retorna el valor de  Q
**/
void descomposicionQR( double **A, int m, int n, double **R, double *vec )
{
	int i, j, k;
	double sum, tmp;

	for( k=0; k<n; k++ ) {

		sum = 0.0;
		for( i=0; i<m; i++ ) {
			tmp = A[i][k];
			sum += tmp * tmp;
		}
		sum = sqrt( sum );
		R[k][k] = sum;
		for( i=0; i<m; i++ )
			vec[i] = A[i][k] / sum;

		for( j=k+1; j<n; j++ ) {
			sum = 0.0;
			for( i=0; i<m; i++ ) 
				sum += vec[i] * A[i][j];

			R[k][j] = sum;

			for( i=0; i<m; i++ ) 
				A[i][j] -= vec[i] * sum;
		}
	}
}


/**
 Si se tiene el sistema Ax = b, A es de tamaño n x n, 
se realiza la matriz aumentada [A|b] y se realiza
la descomposición QR, se obtiene Q[ R | Q^T b ]
Esta rutina obtiene x a partir de la matriz Rp = [ R | Q^T b ]
**/
void resuelve( double **Rp, int n, double *x )
{
	int i, j;
	double suma;

	j = n-1;

	x[j] = Rp[j][n] / Rp[j][j];

	for( i=n-2; i>=0; i-- ) {
		suma = 0.0;
		for( j=i+1; j<n; j++ )
			suma += Rp[i][j] * x[j];

		x[i] = (Rp[i][n] - suma)/Rp[i][i];
	}
}
















// --------------------
// 
void drawPoint( MATRIX *IMG , int X, int Y , int R , int C) 
{
	byte *px;
	for (int i = Y-R; i < Y+R; i++)
	{	
		px = (byte *) IMG-> ptr[i];
		for (int j = X-R; j < X+R; j++){
			*px++ = C;
		}
	}
}



// --------------------
// Find first OBJ pixel in IMG
void findOBJ( const MATRIX *IMG , int* X, int* Y , int COLOR) 
{
	byte *px;
	for (int i = 0; i < IMG-> rows; i++)
	{	
		px = (byte *) IMG-> ptr[i];
		for (int j = 0; j < IMG-> cols; j++){
			if((*px) == COLOR){
				//printf("[%d,%d]= %d  ",i,j,*px);
				*Y=i;
				*X=j;
				return;
			}
			px++;
		}
	}
	*Y=	*X= -1; //No OBJ found!
}

// --------------------
// Negates binary image
void negate( MATRIX *IMG )
{
	byte *px;
	for (int i = 0; i < IMG-> rows; i++)
	{	
		px = (byte *) IMG-> ptr[i];
		for (int j = 0; j < IMG-> cols; j++){
			*px = ((*px) == 255)? 0:255;
			px++;
		}
	}
}

// --------------------
// Clean clean!
void cleanBorder( MATRIX *IN,  int Radius )
{
	int i, j ;
	byte *ptrin;
	/** limpio del borde de la imagen  **/
	
	//Horizontal
    for (int k = 0; k < Radius; ++k)
    {	
    ptrin = (byte *) IN->ptr[0 + k];
    for( j=0; j<IN->cols; j++ ) *ptrin++ = 255;
    
    ptrin = (byte *) IN->ptr[ IN->rows - 1 -k ];
    for( j=0; j<IN->cols; j++ ) *ptrin++ = 255;
    }
	
	//Vertical
	for (int k = 0; k < Radius; ++k)
    {
    for( i=1; i<IN->rows-1; i++ ){
        ptrin = (byte *) IN->ptr[i] + k ;
        *ptrin = 255;
        ptrin = (byte *) IN->ptr[i] + IN->cols - 1 - k ;
        *ptrin = 255;
    }
	}
	
    
}


// --------------------
// Binarizes image given an umbral
void umbral( MATRIX *IMG, int umbral)
{
	byte *px;
	for (int i = 0; i < IMG-> rows; i++)
	{	
		px = (byte *) IMG-> ptr[i];
		for (int j = 0; j < IMG-> cols; j++){
			*px = ((*px) > umbral)? 255:0;
			px++;
		}
	}
}





